%% NPL MCM2MCMC Software: Release 1 - Exponential example 2 

%% Introduction 
%
% This software is concerned with converting a sample from a Bayesian
% posterior distribution corresponding to a particular choice of prior
% distribution derived using the Monte Carlo method (MCM), to a Bayesian
% posterior corresponding to a preferred prior distribution. A
% Metropolis-Hastings Markov Chain Monte Carlo (MCMC) algorithm is used to
% undertake this conversion.
%
% For a given measurement problem, the software performs the following 
% tasks:
%
% 1. Generate an MCM sample from the distribution of the quantity of 
% interest. This distribution can be thought of as a Bayesian posterior 
% distribution corresponding to a particular choice of prior distribution.
%
% 2. Convert the MCM sample into an MCMC sample using a Metropolis-Hastings
% MCMC algorithm. The underlying or target distribution is the posterior
% distribution corresponding to a preferred prior distribution.
%
% 3. Analyse the convergence of the MCMC chains.
%
% 4. Extract and display summary information for the posterior 
% distribution. 

%% Exponential example 2 
%
% These calculations involve the response model
%
% $$ \eta_1 = \alpha_1 {\rm e}^{-\beta}, ~~~ 
%    \zeta_1|\eta_1 \sim {\rm N}(\eta_1,\sigma^2), $$ 
%
% $$ \eta_2 = (\alpha_1 + \alpha_2) {\rm e}^{-\beta}, ~~~ 
%    \zeta_2|\eta_2 \sim {\rm N}(\eta_2,\sigma^2), $$ 
%
% where $$ \beta \sim {\rm N}(b,\sigma_{\rm B}^2) $$ and there is a prior 
% constraint that $$ \alpha_2 \geq 0 $$. 
% 
% The calculations are performed for the case 
% 
% $$ b = 2, ~~~ \sigma = 0.2, ~~~ \sigma_{\rm B} = 0.2, ~~~ 
%    \zeta_1 = 50 {\rm e}^{-b}, ~~~ \zeta_2 = \zeta_1. $$ 
% 
% MCM/MCMC calculations are performed involving this response model 
% assuming a uniform prior for $$ \alpha_1 $$ and $$ \alpha_2 $$ with 
% $$ \alpha_2 \geq 0 $$. Details can be found in the user manual for the 
% software. 

%% Licence agreement
%
% The software is provided with a software user licence agreement and the 
% use of the software is subject to the terms laid out in that agreement. 
% By running the software, the user accepts the terms of the agreement.

%% Main program 

%% 
% Identify the software to the user
  fprintf('\nNPL MCM2MCMC Software: Release 1'); 
  fprintf('\nNational Physical Laboratory, UK'); 
  fprintf('\nApril 2018 \n'); 
  fprintf('\nExponential example 2 \n\n'); 

%% 
% Close all figures and clear all variables
  close all
  clear
  format short
  
%% 
% Set seed for random number generation
  rng(2);
  
%% 
% Assign chain length, number of chains and burn-in period 
  M  = 1100; 
  N  = 100; 
  M0 = 100; 
    
%% 
% Generate MCM sample 

%% 
% Sample influence quantity 
% $$ \beta \sim {\rm N}(b, \sigma_{\rm B}^2) $$ 
  b = 2;
  sigmaB = 0.2;
  B = b + 0.2*randn(M,N);
  
%% 
% Sample responses 
% $$ \eta_1|\zeta_1 \sim {\rm N}(\zeta_1, \sigma^2) $$
  z1 = 50*exp(-b);
  sigma = 0.2;
  Y1 = z1 + sigma*randn(M,N);

%% 
% Sample responses 
% $$ \eta_2|\zeta_2 \sim {\rm N}(\zeta_2, \sigma^2) $$
  z2 = z1 + sigma;
  Y2 = z2 + sigma*randn(M,N);
  
%% 
% Create array to store MCM samples
  A0 = zeros(M,N,3);
  
%% 
% Generate MCM sample corresponding to a propagation of distributions
  A0(:,:,1) = Y1./exp(-B);
  A0(:,:,2) = (Y2-Y1)./exp(-B);
  A0(:,:,3) = B;
  
%% 
% Convert MCM sample to MCMC sample   

%% 
% Record the absolute value of the determinant of the Jacobian matrix of the response model function
  D = exp(-2*B);
  
%% 
% Prior distribution imposing constraint 
  P00 = ones(M,N);
  L00 = A0(:,:,2) < 0;
  P00(L00) = 0;
 
%% 
% Ensure that all chains start at a feasible point 
  L001 = P00(1,:) == 0;

%% 
  if sum(L001) > 0

%% 
% Find a chain with feasible starting point...
    [maxP,imax ] = max(P00(1,:));
    
%% 
% ...and start all infeasible chains at the same point
    A0(1,L001,1) = A0(1,imax,1);
    A0(1,L001,2) = A0(1,imax,2);
    A0(1,L001,3) = A0(1,imax,3);
    D(1,L001) = D(1,imax);
    P00(1,L001) = P00(1,imax);
  end
    
%% 
% Implement MCMC calculations 
  [A,Rhat,Neff,IS,IA] = mcm2mcmc(A0,D,M0,P00);
  
%% 
% Analyse chain convergence   

%% 
% Calculate and display percentage acceptance
  accept = round(100*sum(IA(:))/(M*N)); 
  fprintf('Percentage acceptance: %3g \n\n',accept); 
  
%% 
% Display convergence indices
  fprintf('Convergence indices \n')
  for i = 1:length(Rhat) 
    fprintf('Parameter %1g: %12.6f \n',[i,Rhat(i)])
  end 
  fprintf('\n')
  
%% 
% Display effective numbers of independent draws
  fprintf('Effective number of independent draws \n')
  for i = 1:length(Neff) 
    fprintf('Parameter %1g: %12g \n',[i,round(Neff(i))])
  end 
  fprintf('\n')
    
%% 
% Extract summary information for posterior distribution 
  Q = [0 2.5 50 97.5 100]';
  [abar0,s0,aQ0,V0,AA0] = mcmcsum(A0,0,Q);
  [abar,s,aQ,V,AA] = mcmcsum(A,M0,Q);

%% 
% Display summary information for posterior distribution 
  fprintf('Summary information for posterior distribution \n')
  fprintf('Mean \n'); 
  fprintf('                  MCM          MCMC \n'); 
  for i = 1:length(abar0) 
    fprintf('Parameter %1g: %12g % 12g \n',[i,abar0(i),abar(i)])
  end 
  fprintf('\n')

  fprintf('Standard deviation \n'); 
  fprintf('                  MCM          MCMC \n'); 
  for i = 1:length(abar0) 
    fprintf('Parameter %1g: %12g %12g \n',[i,s0(i),s(i)])
  end 
  fprintf('\n')
  
  for j = 1:size(aQ0, 1) 
    fprintf('%g percentiles \n',Q(j)); 
    fprintf('                  MCM          MCMC \n'); 
    for i = 1:length(abar0) 
      fprintf('Parameter %1g: %12g %12g \n',[i,aQ0(j,i),aQ(j,i)])
    end 
    fprintf('\n')
  end 
  
%% 
% Generate histogram of samples

%% 
% Assign number of bins 
  nB = 30;

%% 
  for j = 1:3

%% 
% Assign histogram for MCM samples 
    a0 = A0(:,:,j);
    a0 = a0(:);
    [na0,aa0] = hist(a0,nB);
    dela0 = aa0(2)-aa0(1);
    na0n = na0/(sum(na0)*dela0);
    yl1 = max(na0n); 

%%
% Assign histogram for MCMC samples 
    [na,aa] = hist(AA(:,j),nB);
    dela = aa(2)-aa(1);
    nan = na/(sum(na)*dela);
    yl2 = max(nan);
    yl = max(yl1,yl2);
    
%% 
% Generate figure 
    figure; 
    bar(aa,nan,'w')
    hold on
    plot(aa0,na0n,'o')
    ylim([0,yl+0.2*yl])
    if j == 1
      xlabel('\alpha_1/1')
      ylabel('p(\alpha_1)/1')
      title('MCM and MCMC samples: \alpha_1')
    elseif j == 2
      xlabel('\alpha_2/1')
      ylabel('p(\alpha_2)/1')
      title('MCM and MCMC samples: \alpha_2')
    elseif j == 3
      xlabel('\beta/1')
      ylabel('p(\beta)/1')
      title('MCM and MCMC samples: \beta')
    end
    legend('MCM','MCMC')

  end

%% References
% 
% [1] A B Forbes, An MCMC algorithm based on GUM Supplement 1 for 
% uncertainty evaluation, Measurement, 2012, 
% DOI:10.1016/j.measurement.2012.01.018
% 
% [2] A B Forbes, A two-stage MCM/MCMC algorithm for uncertainty 
% evaluation, in Advanced Mathematical and Computational Tools for 
% Metrology IX, F Pavese et al., eds., World Scientific, Singpore, 
% pp 159-170, 2012  
% 
% [3] Bayesian Data Analyis, A Gelman, J B Carlin, H S Stern and D B Rubin, 
% Chapman & Hall/CRC, Boca Raton, 2004 (Section 11.6)

%% Acknowledgements
% 
% The work described here was supported by the National Measurement System 
% Directorate of the UK Department for Department for Business, Energy
% and Industrial Strategy.

%% Program identifier
% 
% MCM2MCMC Open-Source Software (Release 1.0): 
% Software for measurement uncertainty evaluation
%
% A B Forbes, K. Jagan and I M Smith, Data Science Group, 
% National Physical Laboratory, UK
%
% Copyright NPL Management Limited 2018